<?php

namespace App\Http\Controllers;

use App\Http\Requests\Post\ApplyRequest;
use App\Http\Requests\Post\CreateRequest;
use App\Http\Requests\Post\EditRequest;
use App\Jobshunt\Mailers\UserMailer;
use App\Jobshunt\Repository\PostsRepositoryInterface;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\File;
use Roumen\Feed\Facades\Feed;

/**
 * @author Abhimanyu Sharma <abhimanyusharma003@gmail.com>
 */
class PostController extends Controller
{
    /**
     * @var PostsRepositoryInterface
     */
    private $post;

    /**
     * @param PostsRepositoryInterface $post
     */
    public function __construct(PostsRepositoryInterface $post)
    {
        $this->middleware('auth', ['only' => ['create', 'store']]);
        $this->post = $post;
    }

    /**
     * @return \BladeView|bool|\Illuminate\View\View
     */
    public function index(Request $request)
    {
        $title = t('Latest Jobs');
        $posts = $this->post->getAll($request);

        return view('posts.list', compact('title', 'posts'));
    }

    /**
     * @param $id
     *
     * @return \BladeView|bool|\Illuminate\View\View
     */
    public function show($id)
    {
        $post = $this->post->getById($id);
        \Event::fire('App\Jobshunt\Events', $post);
        $title = $post->title;

        return view('posts.index', compact('post', 'title'));
    }

    /**
     * @return \BladeView|bool|\Illuminate\View\View
     */
    public function create()
    {
        $title = t('Posting A Job');

        return view('posts.create', compact('title'));
    }

    /**
     * @param Request $request
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(CreateRequest $request)
    {
        $request->session()->put('job.create.data', $request->all());
        $request->session()->save();

        if (env('PAYMENT_PROVIDER') == 'paypal') {
            $post = $this->paypal();
        }
        if (env('PAYMENT_PROVIDER') == 'stripe') {
            $post = $this->stripe($request);
        }

        return redirect()->route('posts.main', ['id' => $post->id, 'slug' => $post->slug])->with('flashSuccess', t('Your job is now posted'));
    }

    /**
     *
     */
    private function paypal()
    {
        $response = $this->setPaypal();
        if ($response->isSuccessful()) {
            $response;
        } elseif ($response->isRedirect()) {
            $response->redirect();
        } else {
            $response->getMessage();
        }
    }

    /**
     * @return mixed
     */
    public function paypalSuccess(Request $request)
    {
        $response = $this->setPaypal('completePurchase');
        $response = $response->getData();
        if (isset($response['PAYMENTINFO_0_ACK']) && $response['PAYMENTINFO_0_ACK'] === 'Success' && !isset($response['L_ERRORCODE0'])) {
            $post = $this->post->create($request->session()->get('job.create.data'), $response['PAYMENTINFO_0_TRANSACTIONID']);

            return redirect()->route('posts.main', ['id' => $post->id, 'slug' => $post->slug])->with('flashSuccess', t('Your job is now posted'));
        }

        return redirect()->route('posts.create')->with('flashError', t('Not able to create post, Please try again'));
    }

    /**
     * @param string $type
     *
     * @return mixed
     */
    private function setPaypal($type = 'purchase')
    {
        $params = [
            'cancelUrl'   => route('home'),
            'returnUrl'   => route('posts.create.paypalSuccess'),
            'name'        => siteSettings('siteName'),
            'description' => 'Job Posting Charges',
            'amount'      => (int)siteSettings('amount') / 100,
            'currency'    => ucwords(siteSettings('currency')),
        ];
        $gateway = \Omnipay\Omnipay::create('PayPal_Express');
        $gateway->setUsername(env('PAYPAL_USERNAME'));
        $gateway->setPassword(env('PAYPAL_PASSWORD'));
        $gateway->setSignature(env('PAYPAL_SIGNATURE'));
        $gateway->setTestMode(true);
        if ($type == 'completePurchase') {
            return $gateway->completePurchase($params)->send();
        }

        return $gateway->purchase($params)->send();
    }

    /**
     * @param $request
     *
     * @return \Illuminate\Http\RedirectResponse|mixed
     */
    private function stripe($request)
    {
        $charge = null;
        if ((int)siteSettings('isFree') == 0) {
            \Stripe\Stripe::setApiKey(config('services.stripe.secret'));
            try {
                $charge = \Stripe\Charge::create([
                        'amount'        => (int)siteSettings('amount'),
                        'currency'      => siteSettings('currency'),
                        'source'        => $request->get('stripeToken'),
                        'description'   => 'Job Posting Charges',
                        'receipt_email' => auth()->user()->email,
                    ]
                );
            } catch (\Stripe\Error\Card $e) {
                return redirect()->back()->with('flashError', t('Your card is declined please use valid card'));
            }
        }
        $post = $this->post->create($request->session()->get('job.create.data'), $charge);

        return $post;
    }

    /**
     * @param $id
     *
     * @return \BladeView|bool|\Illuminate\Contracts\View\Factory|\Illuminate\Http\RedirectResponse|\Illuminate\View\View
     */
    public function getEdit(EditRequest $request)
    {
        $post = $this->post->getById($request->route('id'));
        $title = sprintf('%s - %s', t('Editing'), $post->title);

        return view('posts.edit', compact('post', 'title'));
    }

    /**
     * @param CreateRequest $request
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function postEdit(CreateRequest $request)
    {
        $post = $this->post->edit($request);
        if ($post == null) {
            return redirect()->route('home')->with('flashSuccess', t('Your post is now deleted'));
        }

        return redirect()->route('posts.main', ['id' => $post->id, 'slug' => $post->slug])->with('flashSuccess', t('Your post is now updated'));
    }

    /**
     * @return \BladeView|bool|\Illuminate\View\View
     */
    public function rss(Request $request)
    {
        $posts = $this->post->getAll($request);

        $feed = Feed::make();
        $feed->title = siteSettings('siteName');
        $feed->description = siteSettings('description');
        $feed->logo = asset('favicon.ico');
        $feed->link = $request->fullUrl();
        $feed->setDateFormat('carbon'); // 'datetime', 'timestamp' or 'carbon'
        $feed->pubdate = $posts[0]->created_at;
        $feed->lang = 'en';
        $feed->setShortening(true);
        $feed->setTextLimit(100);
        foreach ($posts as $post) {
            $feed->add($post->title, $post->user->fullname, route('posts.main', ['id' => $post->id, 'slug' => $post->slug]), $post->approved_at, parseDown($post->description), parseDown($post->content));
        }

        return $feed->render('atom');
    }


    public function postApply(ApplyRequest $request, UserMailer $mailer)
    {
        $post = $this->post->getById($request->route('id'));
        $resume = $request->file('resume')->move(public_path() . '/uploads/resumes', sprintf('%s-%s.%s', Carbon::now()->format('d-m-Y-h-m-s'), str_random(), explode('/', $request->file('resume')->getMimeType())[1]));
        $mailer->applyEmail($post->user, $request, $post, sprintf('%s/%s/%s', public_path(), 'uploads/resumes', $resume->getFilename()));
        File::delete(sprintf('%s/%s/%s', public_path(), 'uploads/resumes', $resume->getFilename()));

        return redirect()->back()->with('flashSuccess', t('Your application is now sent'));
    }
}
