<?php

namespace App\Jobshunt\Repository\Eloquent;

use App\Jobshunt\Models\Category;
use App\Jobshunt\Models\Post;
use App\Jobshunt\Models\PostType;
use App\Jobshunt\Repository\PostsRepositoryInterface;
use Carbon\Carbon;
use Illuminate\Http\Request;

/**
 * @author Abhimanyu Sharma <abhimanyusharma003@gmail.com>
 */
class PostsRepository extends AbstractRepository implements PostsRepositoryInterface
{
    /**
     * @var Category
     */
    private $category;
    /**
     * @var PostType
     */
    private $postType;

    /**
     * @param Post $model
     */
    public function __construct(Post $model, Category $category, PostType $postType)
    {
        $this->model = $model;
        $this->category = $category;
        $this->postType = $postType;
    }

    /**
     * @param $id
     * @return mixed
     */
    public function getById($id)
    {
        return $this->model->whereId($id)->with('type', 'category')->firstOrFail();
    }

    /**
     * @return mixed
     */
    public function getAll(Request $request)
    {
        return $this->posts($request)->with('type', 'category')->orderBy('created_at', 'desc')->paginate(perPage());
    }

    /**
     * @param null $type
     * @param array $param
     * @return Post
     */
    private function posts($request = null)
    {
        $post = $this->model->approved();
        if ($request->query('q')) {
            $post = $post->where('title', 'LIKE', '%' . $request->query('q') . '%')
                ->orWhere('city', 'LIKE', '%' . $request->query('q') . '%')
                ->orWhere('state', 'LIKE', '%' . $request->query('q') . '%')
                ->orWhere('country', 'LIKE', '%' . $request->query('q') . '%');
        }
        if ($request->query('category')) {
            if ($category = $this->category->whereSlug($request->query('category'))->first()) {
                $post = $post->whereCategoryId($category->id);
            }
        }
        if ($request->query('type')) {
            if ($type = $this->postType->whereSlug($request->query('type'))->first()) {
                $post = $post->wherePostTypeId($type->id);
            }
        }

        if ($request->query('city')) {
            $post = $post->whereCity($request->query('city'));
        }
        if ($request->query('state')) {
            $post = $post->whereState($request->query('state'));
        }
        if ($request->query('country')) {
            $post = $post->whereCountry($request->query('country'));
        }
        if ($request->query('remote') == true) {
            $post = $post->whereIsRemote(1);
        }
        if ($request->query('tag')) {
            $post = $post->where('tags', 'LIKE', '%' . $request->query('tag') . '%');
        }

        return $post;
    }

    /**
     * @param $request
     * @param $charge
     * @return mixed
     */
    public function create($request, $charge = null)
    {
        $slug = str_slug($request['title']);
        if (!$slug) {
            $slug = str_random(12);
        }
        $post = $this->getNew();
        if ((int)siteSettings('isFree') == 0) {
            if (env('PAYMENT_PROVIDER') == 'paypal') {
                $post->transaction_id = $charge;
            }
            if (env('PAYMENT_PROVIDER') == 'stripe') {
                $post->transaction_id = $charge->id;
            }
        }

        $post->user_id = auth()->user()->id;
        $post->title = $request['title'];
        $post->slug = $slug;
        $post->category_id = $request['category'];
        $post->post_type_id = $request['job_type'];
        if (isset($request['is_remote'])) {
            $post->is_remote = true;
        } elseif (isset($request['location'])) {
            $location = explode(', ', $request['location']);
            if (isset($location[0])) {
                $post->city = $location[0];
            }
            if (isset($location[1]) && isset($location[2])) {
                $post->state = $location[1];
            } elseif (isset($location[1])) {
                $post->country = $location[1];
            }
            if (isset($location[2])) {
                $post->country = $location[2];
            }
            $post->is_remote = false;
            $post->location_id = $request['location_id'];
        } else {
            $post->is_remote = true;
        }
        $post->description = $request['description'];
        if (isset($request['apply_elsewhere'])) {
            $post->apply_elsewhere = 1;
            $post->apply_to = $request['apply_to'];
        }
        if (isset($request['tags'])) {
            $post->tags = implode(',', $request['tags']);
        }
        $post->approved_at = Carbon::now();
        $post->expire_at = Carbon::now()->addDays((int)siteSettings('numberOfDays'));
        $post->save();

        return $post;
    }

    /**
     * @param $request
     * @return mixed
     */
    public function edit($request)
    {
        $post = $this->model->whereId($request->route('id'))->firstOrFail();
        if ($request->get('delete')) {
            $post->delete();

            return null;
        }

        $slug = str_slug($request->get('title'));
        if (!$slug) {
            $slug = str_random(12);
        }
        $post->user_id = auth()->user()->id;
        $post->title = $request->get('title');
        $post->slug = $slug;
        $post->category_id = $request->get('category');
        $post->post_type_id = $request->get('job_type');
        if ($request->get('is_remote')) {
            $post->is_remote = true;
        } else {
            $location = explode(', ', $request->get('location'));
            if (isset($location[0])) {
                $post->city = $location[0];
            }
            if (isset($location[1]) && isset($location[2])) {
                $post->state = $location[1];
            } elseif (isset($location[1])) {
                $post->country = $location[1];
            }
            if (isset($location[2])) {
                $post->country = $location[2];
            }
            $post->is_remote = false;
            $post->location_id = $request->get('location_id');
        }
        $post->description = $request->get('description');
        if ($request->get('apply_elsewhere')) {
            $post->apply_elsewhere = 1;
            $post->apply_to = $request->get('apply_to');
        } else {
            $post->apply_elsewhere = 0;
        }
        if ($request->get('tags')) {
            $post->tags = implode(',', $request->get('tags'));
        }
        $post->approved_at = Carbon::now();
        $post->save();

        return $post;
    }

    /**
     * @param $image
     * @return mixed
     */
    public function incrementViews($post)
    {
        $post->views = $post->views + 1;
        $post->timestamps = false;
        $post->save(['updated_at' => false]);

        return $post;
    }
}
