<?php
/**
 * @author Abhimanyu Sharma <abhimanyusharma003@gmail.com>
 */
namespace App\Jobshunt\Repository\Eloquent;

use App\Jobshunt\Mailers\UserMailer;
use App\Jobshunt\Models\Category;
use App\Jobshunt\Models\Post;
use App\Jobshunt\Models\User;
use App\Jobshunt\Repository\UsersRepositoryInterface;
use Carbon\Carbon;
use Feed;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Session;

class UsersRepository extends AbstractRepository implements UsersRepositoryInterface
{
    /**
     * @param UserMailer $mailer
     * @param User $user
     * @param Image $images
     */
    public function __construct(UserMailer $mailer, User $user, Post $posts, Category $category)
    {
        $this->model = $user;
        $this->posts = $posts;
        $this->mailer = $mailer;
        $this->category = $category;
    }

    /**
     * @param $id
     * @return bool
     */
    public function getById($id)
    {
        $user = $this->model->whereId($id)->firstOrFail();

        return $user;
    }

    /**
     * @param $username
     * @return mixed
     */
    public function getByUsername($username)
    {
        $user = $this->model->whereUsername($username)->firstOrFail();

        return $user;
    }

    /**
     * @return mixed
     */
    public function getAll(Request $request)
    {
        return $this->users($request)->with('category')->orderBy('created_at', 'desc')->paginate(perPage());
    }

    /**
     * @param User $user
     * @return mixed
     */
    public function getUserPosts(User $user)
    {
        $posts = $this->posts->whereUserId($user->id)->paginate(perPage());

        return $posts;
    }

    /**
     * @param Request $request
     * @return bool
     */
    public function createNew(Request $request)
    {
        $activationCode = sha1(str_random(11) . (time() * rand(2, 2000)));

        $this->model->username = $request->get('username');
        $this->model->fullname = $request->get('fullname');
        $this->model->email = $request->get('email');
        $this->model->password = bcrypt($request->get('password'));
        $this->model->email_confirmation = $activationCode;
        $this->model->save();

        $this->mailer->activation($this->model, $activationCode);

        return true;
    }

    /**
     * @param $request
     * @return User|bool
     */
    public function registerViaSocial($request)
    {
        $user = $this->model;
        $user->username = $request->get('username');
        $user->fullname = $request->session()->get($request->route('provider') . '_register')->getName();
        $user->password = bcrypt($request->get('password'));

        if ($request->session()->has('facebook_register')) {
            $user->email = $request->session()->get($request->route('provider') . '_register')->getEmail();
            $user->fbid = $request->session()->get('facebook_register')->getId();
            Session::forget('facebook_register');
        }
        if ($request->session()->has('google_register')) {
            $user->email = $request->session()->get($request->route('provider') . '_register')->getEmail();
            $user->gid = $request->session()->get('google_register')->getId();
            Session::forget('google_register');
        }
        if ($request->session()->has('twitter_register') and session('user_email')) {
            $activationCode = sha1(str_random(11) . (time() * rand(2, 2000)));
            $user->email = session('user_email');
            $user->twid = $request->session()->get('twitter_register')->getId();
            $user->email_confirmation = $activationCode;
            $user->save();
            $this->mailer->activation($user, $activationCode);
            Session::forget('twitter_register');

            return true;
        }
        $user->confirmed_at = Carbon::now();
        $user->save();
        auth()->loginUsingId($user->id);

        return $user;
    }

    /**
     * @param Request $request
     * @return mixed
     */
    public function updateSettings(Request $request)
    {
        $user = $this->model->whereId(auth()->user()->id)->first();
        $user->fullname = $request->get('fullname');
        $user->about = $request->get('about');
        $user->url = $request->get('url');
        $user->tw_url = $request->get('tw_url');
        $user->is_freelancer = $request->get('is_freelancer');
        $user->category_id = $request->get('category');
        if ($request->get('tags')) {
            $user->tags = implode(',', $request->get('tags'));
        }
        $user->save();

        return $user;
    }

    /**
     * @param $username
     * @param $activationCode
     * @return bool
     */
    public function activate($username, $activationCode)
    {
        $user = $this->model->whereUsername($username)->first();
        if ($user && $user->email_confirmation === $activationCode) {
            $user->confirmed_at = Carbon::now();
            $user->save();

            return $user;
        }

        return false;
    }

    /**
     * @param $username
     * @param null $i
     * @return string
     */
    private function getUniqueUserName($username, $i = null)
    {
        $username = str_slug($username . ' ' . $i);
        if (!$username) {
            $username = str_random(5);
        }
        if ($this->model->whereUsername($username)->count()) {
            return $this->getUniqueUserName($username, ++$i);
        }

        return $username;
    }

    /**
     * @param Request $request
     * @return bool
     */
    public function updatePassword(Request $request)
    {
        if (Hash::check($request->get('currentpassword'), $request->user()->password)) {
            $user = auth()->user();
            $user->password = bcrypt($request->get('password'));
            $user->save();

            return true;
        }

        return false;
    }

    private function users($request = null)
    {
        $user = $this->model->freelancer();
        if ($request->query('q')) {
            $user = $user->where('username', 'LIKE', '%' . $request->query('q') . '%')
                ->orWhere('tags', 'LIKE', '%' . $request->query('q') . '%')
                ->orWhere('about', 'LIKE', '%' . $request->query('q') . '%');
        }
        if ($request->query('category')) {
            if ($category = $this->category->whereSlug($request->query('category'))->first()) {
                $user = $user->whereCategoryId($category->id);
            }
        }
        if ($request->query('tag')) {
            $user = $user->where('tags', 'LIKE', '%' . $request->query('tag') . '%');
        }

        return $user;
    }

    public function sendContactEmail($request)
    {
        $description = parseDown($request->get('description'));
        $toUser = $this->model->whereUsername($request->route('username'))->firstOrFail();
        $this->mailer->contactEmail($toUser, auth()->user(), $description);
    }
}
